# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

import subprocess

project = 'coreboot'
copyright = 'CC-by 4.0 the coreboot project'
author = 'the coreboot project'

release = subprocess.check_output(('git', 'describe')).decode("utf-8")
# The short X.Y version.
version = release.split("-")[0]


# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = ["myst_parser"]

myst_heading_anchors = 5
myst_url_schemes = ["http", "https", "mailto", "ftp", "ircs"]

templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']
html_css_files = [
    'theme_overrides.css',  # override wide tables in RTD theme
]



#!/usr/bin/env python3
# Copyright 2022 The ChromiumOS Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Tests for sign_uefi.py.

This is run as part of `make runtests`, or `make runtestscripts` if you
want something a little faster.
"""

from pathlib import Path
import tempfile
import unittest
from unittest import mock

import sign_uefi


class TestSign(unittest.TestCase):
    """Test signing functions in sign_uefi.py."""

    def setUp(self):
        # pylint: disable=consider-using-with
        self.tempdir = tempfile.TemporaryDirectory()
        tempdir = Path(self.tempdir.name)

        # Get key paths.
        self.keys = sign_uefi.Keys(
            private_key=tempdir / "private_key.rsa",
            sign_cert=tempdir / "sign_cert.pem",
            verify_cert=tempdir / "verify_cert.pem",
            kernel_subkey_vbpubk=tempdir / "kernel_subkey.vbpubk",
            crdyshim_private_key=tempdir / "crdyshim.priv.pem",
        )

        # Get target paths.
        self.target_dir = tempdir / "boot"
        self.syslinux_dir = self.target_dir / "syslinux"
        self.efi_boot_dir = self.target_dir / "efi/boot"

        # Make test dirs.
        self.syslinux_dir.mkdir(parents=True)
        self.efi_boot_dir.mkdir(parents=True)

        # Make key files.
        (self.keys.private_key).touch()
        (self.keys.sign_cert).touch()
        (self.keys.verify_cert).touch()
        (self.keys.kernel_subkey_vbpubk).touch()
        (self.keys.crdyshim_private_key).touch()

        # Make EFI files.
        (self.efi_boot_dir / "bootia32.efi").touch()
        (self.efi_boot_dir / "bootx64.efi").touch()
        (self.efi_boot_dir / "testia32.efi").touch()
        (self.efi_boot_dir / "testx64.efi").touch()
        (self.efi_boot_dir / "crdybootia32.efi").touch()
        (self.efi_boot_dir / "crdybootx64.efi").touch()
        (self.syslinux_dir / "vmlinuz.A").touch()
        (self.syslinux_dir / "vmlinuz.B").touch()
        (self.target_dir / "vmlinuz-5.10.156").touch()
        (self.target_dir / "vmlinuz").symlink_to(
            self.target_dir / "vmlinuz-5.10.156"
        )

    def tearDown(self):
        self.tempdir.cleanup()

    @mock.patch("sign_uefi.inject_vbpubk")
    @mock.patch.object(sign_uefi.Signer, "create_detached_signature")
    @mock.patch.object(sign_uefi.Signer, "sign_efi_file")
    def test_sign_target_dir(
        self, mock_sign, mock_detached_sig, mock_inject_vbpubk
    ):
        # Set an EFI glob that matches only some of the EFI files.
        efi_glob = "test*.efi"

        # Sign, but with the actual signing mocked out.
        sign_uefi.sign_target_dir(self.target_dir, self.keys, efi_glob)

        # Check that the correct list of files got signed.
        self.assertEqual(
            mock_sign.call_args_list,
            [
                # The test*.efi files match the glob,
                # the boot*.efi files don't.
                mock.call(self.efi_boot_dir / "testia32.efi"),
                mock.call(self.efi_boot_dir / "testx64.efi"),
                # Two syslinux kernels.
                mock.call(self.syslinux_dir / "vmlinuz.A"),
                mock.call(self.syslinux_dir / "vmlinuz.B"),
                # One kernel in the target dir.
                mock.call(self.target_dir / "vmlinuz-5.10.156"),
            ],
        )

        # Check that `inject_vbpubk` was called on both the crdyboot
        # executables.
        self.assertEqual(
            mock_inject_vbpubk.call_args_list,
            [
                mock.call(self.efi_boot_dir / "crdybootia32.efi", self.keys),
                mock.call(self.efi_boot_dir / "crdybootx64.efi", self.keys),
            ],
        )

        # Check that `create_detached_signature` was called on both
        # the crdyboot executables.
        self.assertEqual(
            mock_detached_sig.call_args_list,
            [
                mock.call(self.efi_boot_dir / "crdybootia32.efi"),
                mock.call(self.efi_boot_dir / "crdybootx64.efi"),
            ],
        )

    @mock.patch("sign_uefi.inject_vbpubk")
    @mock.patch.object(sign_uefi.Signer, "create_detached_signature")
    @mock.patch.object(sign_uefi.Signer, "sign_efi_file")
    def test_no_crdyshim_key(
        self, _mock_sign, _mock_detached_sig, _mock_inject_vbpubk
    ):
        """Test for older keysets that don't have the crdyshim key."""
        self.keys.crdyshim_private_key.unlink()

        # Error: crdyboot files are supposed to be signed, but the
        # crdyshim key isn't present.
        with self.assertRaises(SystemExit):
            sign_uefi.sign_target_dir(
                self.target_dir, self.keys, "crdyboot*.efi"
            )

        # Success: the crdyboot files aren't present, so the crdyshim
        # key is not required.
        (self.efi_boot_dir / "crdybootia32.efi").unlink()
        (self.efi_boot_dir / "crdybootx64.efi").unlink()
        sign_uefi.sign_target_dir(self.target_dir, self.keys, "crdyboot*.efi")

    @mock.patch.object(sign_uefi.Signer, "sign_efi_file")
    def test_sign_target_file(self, mock_sign):
        # Test signing a specific file.
        sign_uefi.sign_target_file(
            self.efi_boot_dir / "bootia32.efi", self.keys
        )

        # Check that we made the expected signer call.
        self.assertIn(
            [
                mock.call(self.efi_boot_dir / "bootia32.efi"),
            ],
            mock_sign.call_args_list,
        )

    @mock.patch("sign_uefi.subprocess.run")
    def test_inject_vbpubk(self, mock_run):
        efi_file = self.efi_boot_dir / "crdybootx64.efi"
        sign_uefi.inject_vbpubk(efi_file, self.keys)

        # Check that the expected command runs.
        self.assertEqual(
            mock_run.call_args_list,
            [
                mock.call(
                    [
                        "sudo",
                        "objcopy",
                        "--update-section",
                        f".vbpubk={self.keys.kernel_subkey_vbpubk}",
                        efi_file,
                    ],
                    check=True,
                )
            ],
        )

    @mock.patch("sign_uefi.subprocess.run")
    def test_create_detached_signature(self, mock_run):
        with tempfile.TemporaryDirectory() as tempdir:
            tempdir = Path(tempdir)
            signer = sign_uefi.Signer(tempdir, self.keys)

            efi_file = self.efi_boot_dir / "crdybootx64.efi"
            signer.create_detached_signature(efi_file)

            # Check that the expected commands run.
            self.assertEqual(
                mock_run.call_args_list,
                [
                    mock.call(
                        [
                            "openssl",
                            "pkeyutl",
                            "-sign",
                            "-rawin",
                            "-in",
                            efi_file,
                            "-inkey",
                            self.keys.crdyshim_private_key,
                            "-out",
                            tempdir / "crdybootx64.sig",
                        ],
                        check=True,
                    ),
                    mock.call(
                        [
                            "sudo",
                            "cp",
                            tempdir / "crdybootx64.sig",
                            self.efi_boot_dir / "crdybootx64.sig",
                        ],
                        check=True,
                    ),
                ],
            )


class TestUtils(unittest.TestCase):
    """Test utility functions in sign_uefi.py."""

    def test_is_pkcs11_key_path(self):
        self.assertFalse(sign_uefi.is_pkcs11_key_path(Path("private_key.rsa")))

        self.assertTrue(
            sign_uefi.is_pkcs11_key_path("pkcs11:object=private_key")
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# Copyright 2022 The ChromiumOS Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Sign the UEFI binaries in the target directory.

The target directory can be either the root of ESP or /boot of root filesystem.
"""

import argparse
import dataclasses
import logging
import os
from pathlib import Path
import shutil
import subprocess
import sys
import tempfile
from typing import List, Optional


def ensure_executable_available(name):
    """Exit non-zero if the given executable isn't in $PATH.

    Args:
        name: An executable's file name.
    """
    if not shutil.which(name):
        sys.exit(f"Cannot sign UEFI binaries ({name} not found)")


def ensure_file_exists(path, message):
    """Exit non-zero if the given file doesn't exist.

    Args:
        path: Path to a file.
        message: Error message that will be printed if the file doesn't exist.
    """
    if not path.is_file():
        sys.exit(f"{message}: {path}")


def is_pkcs11_key_path(path: os.PathLike) -> bool:
    """Check if the key path is a PKCS#11 URI.

    If the key path starts with "pkcs11:", it should be treated as a
    PKCS#11 URI instead of a local file path.
    """
    return str(path).startswith("pkcs11:")


@dataclasses.dataclass(frozen=True)
class Keys:
    """Public and private keys paths.

    Attributes:
        private_key: Path of the private signing key
        sign_cert: Path of the signing certificate
        verify_cert: Path of the verification certificate
        kernel_subkey_vbpubk: Path of the kernel subkey public key
        crdyshim_private_key: Path of the private crdyshim key
    """

    private_key: os.PathLike
    sign_cert: os.PathLike
    verify_cert: os.PathLike
    kernel_subkey_vbpubk: os.PathLike
    crdyshim_private_key: os.PathLike


class Signer:
    """EFI file signer.

    Attributes:
        temp_dir: Path of a temporary directory used as a workspace.
        keys: An instance of Keys.
    """

    def __init__(self, temp_dir: os.PathLike, keys: Keys):
        self.temp_dir = temp_dir
        self.keys = keys

    def sign_efi_file(self, target):
        """Sign an EFI binary file, if possible.

        Args:
            target: Path of the file to sign.
        """
        logging.info("signing efi file %s", target)

        # Remove any existing signatures, in case the file being signed
        # was signed previously. Allow this to fail, as there may not be
        # any signatures.
        subprocess.run(["sudo", "sbattach", "--remove", target], check=False)

        signed_file = self.temp_dir / target.name
        sign_cmd = [
            "sbsign",
            "--key",
            self.keys.private_key,
            "--cert",
            self.keys.sign_cert,
            "--output",
            signed_file,
            target,
        ]
        if is_pkcs11_key_path(self.keys.private_key):
            sign_cmd += ["--engine", "pkcs11"]

        try:
            logging.info("running sbsign: %r", sign_cmd)
            subprocess.run(sign_cmd, check=True)
        except subprocess.CalledProcessError:
            logging.warning("cannot sign %s", target)
            return

        subprocess.run(
            ["sudo", "cp", "--force", signed_file, target], check=True
        )
        try:
            subprocess.run(
                ["sbverify", "--cert", self.keys.verify_cert, target],
                check=True,
            )
        except subprocess.CalledProcessError:
            sys.exit("Verification failed")

    def create_detached_signature(self, input_path: os.PathLike):
        """Create a detached signature using the crdyshim private key.

        The signature file will be created at the same location as
        |efi_file|, but with the extension changed to ".sig".

        Args:
            input_path: Path of the file to sign.
        """
        sig_name = input_path.stem + ".sig"

        # Create the signature in the temporary dir so that openssl
        # doesn't have to run as root.
        temp_sig_path = self.temp_dir / sig_name
        cmd = [
            "openssl",
            "pkeyutl",
            "-sign",
            "-rawin",
            "-in",
            input_path,
            "-inkey",
            self.keys.crdyshim_private_key,
            "-out",
            temp_sig_path,
        ]
        if is_pkcs11_key_path(self.keys.private_key):
            cmd += ["--engine", "pkcs11"]

        logging.info("creating signature: %r", cmd)
        subprocess.run(cmd, check=True)

        output_path = input_path.parent / sig_name
        subprocess.run(["sudo", "cp", temp_sig_path, output_path], check=True)


def inject_vbpubk(efi_file: os.PathLike, keys: Keys):
    """Update a UEFI executable's vbpubk section.

    The crdyboot bootloader contains an embedded public key in the
    ".vbpubk" section. This function replaces the data in the existing
    section (normally containing a dev key) with the real key.

    Args:
        efi_file: Path of a UEFI file.
        keys: An instance of Keys.
    """
    section_name = ".vbpubk"
    logging.info("updating section %s in %s", section_name, efi_file.name)
    subprocess.run(
        [
            "sudo",
            "objcopy",
            "--update-section",
            f"{section_name}={keys.kernel_subkey_vbpubk}",
            efi_file,
        ],
        check=True,
    )


def check_keys(keys: Keys):
    """Checks existence of the keys used for signing.

    Exits the process if the check fails and a key is
    not present.

    Args:
        keys: The keys to check.
    """

    # Check for the existence of the key files.
    ensure_file_exists(keys.verify_cert, "No verification cert")
    ensure_file_exists(keys.sign_cert, "No signing cert")
    ensure_file_exists(keys.kernel_subkey_vbpubk, "No kernel subkey public key")
    # Only check the private keys if they are local paths rather than a
    # PKCS#11 URI.
    if not is_pkcs11_key_path(keys.private_key):
        ensure_file_exists(keys.private_key, "No signing key")
    # Do not check |keys.crdyshim_private_key| here, as it is not
    # present in all key set versions.


def sign_target_dir(target_dir: os.PathLike, keys: Keys, efi_glob: str):
    """Sign various EFI files under |target_dir|.

    Args:
        target_dir: Path of a boot directory. This can be either the
            root of the ESP or /boot of the root filesystem.
        keys: An instance of Keys.
        efi_glob: Glob pattern of EFI files to sign, e.g. "*.efi".
    """
    bootloader_dir = target_dir / "efi/boot"
    syslinux_dir = target_dir / "syslinux"
    kernel_dir = target_dir

    # Verify all keys are present for signing.
    check_keys(keys)

    with tempfile.TemporaryDirectory() as working_dir:
        working_dir = Path(working_dir)
        signer = Signer(working_dir, keys)

        for efi_file in sorted(bootloader_dir.glob(efi_glob)):
            if efi_file.is_file():
                signer.sign_efi_file(efi_file)

        for efi_file in sorted(bootloader_dir.glob("crdyboot*.efi")):
            # This key is required to create the detached signature.
            ensure_file_exists(
                keys.crdyshim_private_key, "No crdyshim private key"
            )

            if efi_file.is_file():
                inject_vbpubk(efi_file, keys)
                signer.create_detached_signature(efi_file)

        for syslinux_kernel_file in sorted(syslinux_dir.glob("vmlinuz.?")):
            if syslinux_kernel_file.is_file():
                signer.sign_efi_file(syslinux_kernel_file)

        kernel_file = (kernel_dir / "vmlinuz").resolve()
        if kernel_file.is_file():
            signer.sign_efi_file(kernel_file)


def sign_target_file(target_file: os.PathLike, keys: Keys):
    """Signs a single EFI file.

    Args:
        target_file: Path a file to sign.
        keys: An instance of Keys.
    """

    # Verify all keys are present for signing.
    check_keys(keys)

    with tempfile.TemporaryDirectory() as working_dir:
        working_dir = Path(working_dir)
        signer = Signer(working_dir, keys)

        if target_file.is_file():
            signer.sign_efi_file(target_file)
        else:
            sys.exit("File not found")


def get_parser() -> argparse.ArgumentParser:
    """Get CLI parser."""
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "--target-dir",
        type=Path,
        help="Path of a boot directory, either the root of the ESP or "
        "/boot of the root filesystem",
        required=False,
    )
    parser.add_argument(
        "--target-file",
        type=Path,
        help="Path of an EFI binary file to sign",
        required=False,
    )
    parser.add_argument(
        "--private-key",
        type=Path,
        help="Path of the private signing key",
        required=True,
    )
    parser.add_argument(
        "--sign-cert",
        type=Path,
        help="Path of the signing certificate",
        required=True,
    )
    parser.add_argument(
        "--verify-cert",
        type=Path,
        help="Path of the verification certificate",
        required=True,
    )
    parser.add_argument(
        "--kernel-subkey-vbpubk",
        type=Path,
        help="Path of the kernel subkey public key",
        required=True,
    )
    parser.add_argument(
        "--crdyshim-private-key",
        type=Path,
        help="Path of the crdyshim private key",
        required=True,
    )
    parser.add_argument(
        "--efi-glob",
        help="Glob pattern of EFI files to sign, e.g. '*.efi'",
        required=False,
    )
    return parser


def main(argv: Optional[List[str]] = None) -> Optional[int]:
    """Sign UEFI binaries.

    Args:
        argv: Command-line arguments.
    """
    logging.basicConfig(level=logging.INFO)

    parser = get_parser()
    opts = parser.parse_args(argv)

    for tool in (
        "objcopy",
        "sbattach",
        "sbsign",
        "sbverify",
    ):
        ensure_executable_available(tool)

    keys = Keys(
        private_key=opts.private_key,
        sign_cert=opts.sign_cert,
        verify_cert=opts.verify_cert,
        kernel_subkey_vbpubk=opts.kernel_subkey_vbpubk,
        crdyshim_private_key=opts.crdyshim_private_key,
    )

    if opts.target_dir:
        if not opts.efi_glob:
            sys.exit("Unable to run: specify '--efi-glob'")
        sign_target_dir(opts.target_dir, keys, opts.efi_glob)
    elif opts.target_file:
        sign_target_file(opts.target_file, keys)
    else:
        sys.exit(
            "Unable to run, either provide '--target-dir' or '--target-file'"
        )


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



#!/usr/bin/env python3
# Copyright 2024 The ChromiumOS Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Tests for generate_android_cloud_config.py.

This is run as part of `make runtests`, or `make runtestscripts` if you
want something a little faster.
"""

import os
import unittest

from generate_android_cloud_config import _parse_flags
from generate_android_cloud_config import _validate
from generate_android_cloud_config import PKCS11_MODULE_PATH


PKCS11_TEST_PATH = "test_path"


class Test(unittest.TestCase):
    """Basic unit test cases for generate_android_cloud_config.py"""

    def test_input_args_default(self):
        """Test default input arguments"""
        args = _parse_flags([""])
        self.assertEqual(args.output_dir, os.getcwd())

    def test_validate_missing_pkcs11_module_path(self):
        with self.assertRaises(SystemExit):
            output_dir = _validate(_parse_flags([""]))


if __name__ == "__main__":
    unittest.main()
